#!/usr/bin/env python
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import subprocess
import sys


def helm_build_package(repodir, srcdir):
    command_line = "cd %s; helm package %s" % (repodir, srcdir)
    try:
        res = subprocess.check_output(
            command_line, shell=True,
            executable='/bin/bash')
        res = res.strip()  # strip whitespace
        if res != "":
            print(res)
    except subprocess.CalledProcessError as e:
        print(e)
        raise


def _isdir(path, entry):
    return os.path.isdir(os.path.join(path, entry))


def main():
    path = os.path.abspath(os.path.dirname(sys.argv[0]))
    srcdir = os.path.join(path, "../helm")
    if len(sys.argv) < 2:
        sys.stderr.write("You must specify the repo directory to build in.\n")
        sys.exit(-1)
    repodir = sys.argv[1]
    if not os.path.isdir(repodir):
        sys.stderr.write("The specified repo directory does not exist.\n")
        sys.exit(-1)

    microdir = os.path.join(srcdir, "microservice")
    microservices = os.listdir(microdir)

    packages = [p for p in microservices if _isdir(microdir, p)]
    count = 1
    for package in packages:
        if sys.stdout.isatty():
            sys.stdout.write("\rProcessing %i/%i" % (count, len(packages)))
            sys.stdout.flush()
            count += 1
        helm_build_package(repodir, os.path.join(microdir, package))
    if sys.stdout.isatty():
            sys.stdout.write("\r                             \n")
            sys.stdout.flush()

if __name__ == '__main__':
    sys.exit(main())



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import os
import yaml

from kolla_kubernetes.tests import base
from kolla_kubernetes.utils import ExecUtils


def _isdir(path, entry):
    return os.path.isdir(os.path.join(path, entry))


class TestK8sTemplatesTest(base.BaseTestCase):

    def _validate_image_pull_policy(self, package, pod):
        for container in pod['spec']['containers']:
            if 'imagePullPolicy' not in container:
                raise Exception("imagePullPolicy not in %s" % package)

    def test_validate_templates(self):
        srcdir = os.environ['HELMDIR']
        helmbin = os.environ['HELMBIN']
        repodir = os.environ['REPODIR']
        microdir = os.path.join(srcdir, "microservice")
        microservices = os.listdir(microdir)
        packages = [p for p in microservices if _isdir(microdir, p)]
        print("Working on:")
        for package in packages:
            print("    %s" % package)
            with open(os.path.join(microdir, package, 'Chart.yaml')) as stream:
                version = yaml.safe_load(stream)['version']

            cmd = "%s template %s/%s-%s.tgz" % (helmbin, repodir,
                                                package, version)
            out, err = ExecUtils.exec_command(cmd)
            if err:
                raise err

            l = yaml.safe_load_all(out)
            for y in l:
                js = '[]'
                try:
                    # If there is a beta init container, validate it is proper
                    # json
                    key = 'pod.beta.kubernetes.io/init-containers'
                    js = y['spec']['template']['metadata']['annotations'][key]
                except KeyError:
                    pass
                except TypeError as e:
                    m = ("'NoneType' object has no attribute '__getitem__'",
                         "'NoneType' object is not subscriptable")
                    if e.args[0] not in m:
                        raise
                json.loads(js)
                pod = None
                try:
                    pod = y['spec']['template']
                except Exception:
                    pass
                if pod:
                    self._validate_image_pull_policy(package, pod)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

from kolla_kubernetes.commands import base_command
from kolla_kubernetes import service_resources


KKR = service_resources.KollaKubernetesResources.Get()


class _ServiceCommand(base_command.KollaKubernetesBaseCommand):

    _action = None  # must be set in derived classes

    def get_parser(self, prog_name):
        parser = super(_ServiceCommand, self).get_parser(prog_name)
        parser.add_argument(
            "service_name",
            metavar="<service-name>",
            help=("One of [%s]" % ("|".join(KKR.getServices().keys())))
        )
        return parser

    def take_action(self, args):
        assert self._action is not None, (
            "code error: derived classes must set _action")

        if args.service_name not in KKR.getServices().keys():
            msg = ("service_name [{}] not in valid service_names [{}]".format(
                args.service_name,
                "|".join(KKR.getServices().keys())))
            raise Exception(msg)

        service = KKR.getServiceByName(args.service_name)
        if (self._action == 'bootstrap'):
            service.do_apply(
                'create', service_resources.Service.LEGACY_BOOTSTRAP_RESOURCES)
        elif (self._action == 'run'):
            service.do_apply('create',
                             service_resources.Service.LEGACY_RUN_RESOURCES)
        elif (self._action == 'kill'):
            service.do_apply('delete',
                             service_resources.Service.VALID_RESOURCE_TYPES)
        else:
            raise Exception("Code Error")


class Bootstrap(_ServiceCommand):
    """Roll out configurations and bootstrap a service."""
    _action = 'bootstrap'


class Run(_ServiceCommand):
    """Run a service."""
    _action = 'run'


class Kill(_ServiceCommand):
    """Kill a service."""
    _action = 'kill'



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import yaml

from kolla_kubernetes.commands.cmd_resource import ResourceTemplate
from kolla_kubernetes.service_resources import KollaKubernetesResources
from kolla_kubernetes.tests import base

KKR = KollaKubernetesResources.Get()
# FIXME configmap should be here, but requires config to be done in tox.
# as it seems to change home.
RESOURCE_TYPES = ('secret pv pvc svc bootstrap pod').split(" ")

technical_debt = {
    'nodeSelector': [],
    'namespaceNotFound': [],
    'namespaceInTemplate': [],
    'namespaceHardCoded': [],
    'nameInTemplateMetadata': [],
    'mainContainer': [],
    'resourceNameObjNameNoMatch': [],
    'typeInName': []
}


def unknown_technical_debt(args, selector):
    for i in technical_debt[selector]:
        if i[0] == args.resource_type and i[1] == args.service_name and \
           i[2] == args.resource_name[0]:
            return False
    return True


class argobj(object):

    def __init__(self, action, resource_type, service_name, resource_name):
        self.service_name = service_name
        self.resource_type = resource_type
        self.resource_name = [resource_name]
        self.action = action
        self.print_jinja_keys_regex = None
        self.print_jinja_vars = False
        self.debug_container = None


def on_each_template(func):
    for service_name in KKR.getServices():
        service = KKR.getServiceByName(service_name)
        for resource_type in RESOURCE_TYPES:
            templates = service.getResourceTemplatesByType(resource_type)
            for template in templates:
                template_name = template.getName()
                args = argobj('create',
                              resource_type,
                              service_name,
                              template_name)
                print("Processing:", resource_type,
                      service_name, template_name)
                rt = ResourceTemplate('kolla-kubernetes.py',
                                      '', 'resource-template')
                o = rt.take_action(args=args, skip_and_return=True)
                func(args, o)


class TestTemplatesTest(base.BaseTestCase):

    def test_validate_names(self):
        service_names = {}
        template_names = {}
        for service_name in KKR.getServices():
            service_names[service_name] = True
            service = KKR.getServiceByName(service_name)
            for resource_type in RESOURCE_TYPES:
                tnprt = template_names.get(resource_type)
                if tnprt is None:
                    template_names[resource_type] = tnprt = {}
                templates = service.getResourceTemplatesByType(resource_type)
                for template in templates:
                    template_name = template.getName()
                    args = argobj('create',
                                  resource_type,
                                  service_name,
                                  template_name)
                    for part in template_name.split('-'):
                        if part.lower() in ('petset', 'deployment', 'job',
                                            'replicationcontroller', 'pod',
                                            'daemonset', 'configmap',
                                            'secret', 'configmap',
                                            'statefulset',
                                            'ps', 'pv', 'pvc', 'disk',
                                            'ds', 'persistentvolume',
                                            'persistentvolumeclaim') and \
                           unknown_technical_debt(args, 'typeInName'):
                            raise Exception("type in name. [%s]" % part)
                    if service_names.get(template_name, False) and \
                        len(templates) != 1 and resource_type != 'svc':
                        s = "Resource name %s matches service name and" \
                            " there are more then one resource." \
                            % template_name
                        raise Exception(s)
                    if tnprt.get(template_name, False):
                        s = "Resource name %s matches another template name" \
                            % template_name
                        raise Exception(s)
                    tnprt[template_name] = True
                    if not template_name.startswith("%s-" % service_name) and \
                       template_name != service_name:
                        m = "%s doesn't start with %s-" % (template_name,
                                                           service_name)
                        raise Exception(m)

    def test_validate_templates(self):
        WERROR = True
        WARNING = {'found': False}

        def func(args, o):
            # Check if template is yaml
            y = yaml.safe_load(o)
            js = '[]'
            try:
                # If there is an alpha init container, validate it is proper
                # json
                key = 'pod.alpha.kubernetes.io/init-containers'
                js = y['spec']['template']['metadata']['annotations'][key]
            except KeyError:
                pass
            except TypeError as e:
                m = ("'NoneType' object has no attribute '__getitem__'",
                     "'NoneType' object is not subscriptable")
                if e.args[0] not in m:
                    raise
            json.loads(js)
            if args.service_name != 'ceph':
                kind = y['kind']
                if 'namespace' not in y['metadata'] and \
                   kind != 'PersistentVolume' and \
                   unknown_technical_debt(args, 'namespaceNotFound'):
                    raise Exception("namespace not found but required.")
                if 'namespace' in y['metadata'] and \
                   y['metadata']['namespace'] != 'not_real_namespace' and \
                   unknown_technical_debt(args, 'namespaceHardCoded'):
                    raise Exception("namespace is hardcoded.")
                if y['metadata']['name'] != args.resource_name[0] and \
                   unknown_technical_debt(args, 'resourceNameObjNameNoMatch'):
                    raise Exception("Object name does not match the" +
                                    " resource_name.")
                if kind in ('PetSet', 'Deployment', 'Job', 'DaemonSet',
                            'ReplicationController', 'Pod'):
                    pod = y
                    if kind != 'Pod':
                        pod = y['spec']['template']
                    if 'nodeSelector' not in pod['spec'] and \
                       unknown_technical_debt(args, 'nodeSelector'):
                        raise Exception("nodeSelector not found but required.")
                    if 'metadata' in pod and 'namespace' in pod['metadata'] and \
                       kind != 'Pod' and \
                       unknown_technical_debt(args, 'namespaceInTemplate'):
                        raise Exception("namespace found in inner template." +
                                        " Its redundant.")
                    if 'metadata' in pod and 'name' in pod['metadata'] and \
                       kind != 'Pod' and \
                       unknown_technical_debt(args, 'nameInTemplateMetadata'):
                        raise Exception("name in pod metadata. Its generated" +
                                        " from the main metadata. It can" +
                                        " cause issues.")
                    main_found = False
                    for container in pod['spec']['containers']:
                        if container['name'] == 'main':
                            main_found = True
                    container_count = len(pod['spec']['containers'])
                    if ((kind == 'Job' and container_count == 1) or
                        kind != 'Job') and not main_found and \
                       unknown_technical_debt(args, 'mainContainer'):
                        raise Exception("Pod does not contain a container" +
                                        " named main.")
        on_each_template(func)
        if WARNING['found'] and WERROR:
            raise Exception('Found Warning when Werror set.')



#!/usr/bin/env python
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
import os
import subprocess
import sys
import yaml


stateful_services = [
    'rabbitmq-pv',
    'rabbitmq-pvc',
    'mariadb-pv',
    'mariadb-pvc',
    'glance-pv',
    'glance-pvc',
    'helm-repo-pv',
    'helm-repo-pvc'
]

pod_http_termination = [
    'neutron-server-deployment',
    'nova-api-deployment',
    'nova-novncproxy-deployment',
    'nova-placement-deployment',
    'cinder-api-deployment',
    'glance-api-deployment',
    'glance-registry-deployment',
    'heat-api-cfn-deployment',
    'heat-api-deployment',
    'ironic-api-deployment',
    'ironic-inspector-deployment'
]

common_mariadb = [
    'keystone-create-db-job',
    'neutron-create-db-job',
    'nova-create-db-job',
    'nova-api-create-db-job',
    'nova-cell0-create-db-job',
    'cinder-create-db-job',
    'glance-create-db-job',
    'ironic-api-create-db-job',
    'ironic-api-manage-db-job',
    'ironic-inspector-create-db-job',
    'ironic-inspector-manage-db-job',
    'heat-create-db-job',
    'cinder-delete-db-job',
    'glance-delete-db-job',
    'keystone-delete-db-job',
    'neutron-delete-db-job',
    'nova-delete-db-job',
    'nova-api-delete-db-job',
    'ironic-api-delete-db-job',
    'ironic-inspector-delete-db-job',
    'heat-create-db-job',
    'heat-delete-db-job'
]

common_create_keystone_admin = [
    'ironic-create-keystone-service-job',
    'ironic-create-keystone-user-job',
    'ironic-api-create-keystone-endpoint-public-job',
    'ironic-api-create-keystone-endpoint-internal-job',
    'ironic-api-create-keystone-endpoint-admin-job',
    'ironic-inspector-create-keystone-service-job',
    'ironic-inspector-create-keystone-user-job',
    'ironic-inspector-create-keystone-endpoint-public-job',
    'ironic-inspector-create-keystone-endpoint-internal-job',
    'ironic-inspector-create-keystone-endpoint-admin-job',
    'neutron-create-keystone-service-job',
    'neutron-create-keystone-endpoint-public-job',
    'neutron-create-keystone-endpoint-internal-job',
    'neutron-create-keystone-endpoint-admin-job',
    'cinder-create-keystone-user-job',
    'glance-create-keystone-user-job',
    'glance-create-keystone-service-job',
    'glance-create-keystone-endpoint-public-job',
    'glance-create-keystone-endpoint-internal-job',
    'glance-create-keystone-endpoint-admin-job',
    'neutron-create-keystone-user-job',
    'nova-create-keystone-user-job',
    'nova-create-keystone-service-job',
    'nova-create-keystone-endpoint-public-job',
    'nova-create-keystone-endpoint-internal-job',
    'nova-create-keystone-endpoint-admin-job',
    'nova-placement-create-keystone-user-job',
    'nova-placement-create-keystone-service-job',
    'nova-placement-create-keystone-endpoint-public-job',
    'nova-placement-create-keystone-endpoint-internal-job',
    'nova-placement-create-keystone-endpoint-admin-job',
    'cinder-create-keystone-service-job',
    'cinder-create-keystone-servicev2-job',
    'cinder-create-keystone-endpoint-public-job',
    'cinder-create-keystone-endpoint-internal-job',
    'cinder-create-keystone-endpoint-admin-job',
    'cinder-create-keystone-endpoint-publicv2-job',
    'cinder-create-keystone-endpoint-internalv2-job',
    'cinder-create-keystone-endpoint-adminv2-job',
    'cinder-delete-keystone-service-job',
    'cinder-delete-keystone-servicev2-job',
    'cinder-delete-keystone-user-job',
    'glance-delete-keystone-service-job',
    'glance-delete-keystone-user-job',
    'neutron-delete-keystone-service-job',
    'neutron-delete-keystone-user-job',
    'nova-delete-keystone-service-job',
    'nova-delete-keystone-user-job',
    'ironic-inspector-delete-keystone-service-job',
    'ironic-inspector-delete-keystone-user-job',
    'ironic-delete-keystone-service-job',
    'ironic-delete-keystone-user-job',
    'heat-create-keystone-user-job',
    'heat-create-keystone-service-job',
    'heat-create-keystone-endpoint-public-job',
    'heat-create-keystone-endpoint-internal-job',
    'heat-create-keystone-endpoint-admin-job',
    'heat-delete-keystone-user-job',
    'heat-delete-keystone-service-job',
    'heat-cfn-create-keystone-service-job',
    'heat-cfn-create-keystone-endpoint-public-job',
    'heat-cfn-create-keystone-endpoint-internal-job',
    'heat-cfn-create-keystone-endpoint-admin-job',
    'heat-cfn-delete-keystone-service-job'
]


def helm_dep_up(srcdir):
    command_line = "cd %s; helm dep up --skip-refresh" % (srcdir)
    try:
        res = subprocess.check_output(
            command_line, shell=True,
            executable='/bin/bash')
        res = res.strip()  # strip whitespace
        if res != "":
            print(res)
    except subprocess.CalledProcessError as e:
        print(e)
        raise


def _isdir(path, entry):
    return os.path.isdir(os.path.join(path, entry))


def merge_dict(a, b):
    for key in b:
        if key not in a:
            a[key] = b[key]
        else:
            if isinstance(b[key], dict):
                merge_dict(a[key], b[key])
            else:
                a[key] = b[key]


def main():
    path = os.path.abspath(os.path.dirname(sys.argv[0]))

    srcdir = os.path.join(path, "..", "helm")
    microdir = os.path.join(srcdir, "microservice")
    microservices = os.listdir(microdir)
    values = yaml.safe_load(open(os.path.join(srcdir, "all_values.yaml")))

    packages = [p for p in microservices if _isdir(microdir, p)]
    count = 1
    for package in packages:
        if sys.stdout.isatty():
            sys.stdout.write("\rProcessing %i/%i" % (count, len(packages)))
            sys.stdout.flush()
            count += 1
        pkgdir = os.path.join(microdir, package)
        helm_dep_up(pkgdir)
        pkg_values = copy.deepcopy(values['common'])
        if package in common_create_keystone_admin:
            key = 'common-create-keystone-admin'
            merge_dict(pkg_values, values[key])
        if package in pod_http_termination:
            merge_dict(pkg_values, values['pod-http-termination'])
        if package in stateful_services:
            merge_dict(pkg_values, values['stateful-service'])
        if package in common_mariadb:
            merge_dict(pkg_values, values['common-mariadb'])
        if package in values:
            merge_dict(pkg_values, values[package])
        with open(os.path.join(microdir, package, "values.yaml"), "w") as f:
            f.write("# This file is generated. Please edit all_values.yaml\n")
            f.write("# and rerun tools/helm_prebuild.py\n")
            f.write(yaml.safe_dump(pkg_values, default_flow_style=False))
    if sys.stdout.isatty():
            sys.stdout.write("\r                             \n")
            sys.stdout.flush()

if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
import json


def process(json_file, key):
    try:
        with open(json_file) as _json:
            j = json.load(_json)
        a = []
        for v in j['config_files']:
            if v['source'] == '/var/lib/kolla/config_files/ceph.*':
                i = copy.deepcopy(v)
                f = "/var/lib/kolla/config_files/ceph.client.%s.keyring" % key
                i['source'] = f
                i['dest'] = "/etc/ceph/ceph.client.%s.keyring" % key
                a.append(i)
                i = copy.deepcopy(v)
                i['source'] = '/var/lib/kolla/config_files/ceph.conf'
                i['dest'] = '/etc/ceph/ceph.conf'
                a.append(i)
            else:
                a.append(v)
        if len(a) != len(j['config_files']):
            j['config_files'] = a
            with open(json_file, 'w') as f:
                f.write(json.dumps(j, indent=4, separators=(',', ': ')))
    except Exception:
        pass

process('/etc/kolla/glance-api/config.json', 'glance')
process('/etc/kolla/cinder-volume/config.json', 'cinder')
process('/etc/kolla/nova-compute/config.json', 'nova')



#!/usr/bin/env python
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os.path
import six
import subprocess
import sys
import yaml

from kolla_kubernetes.service_resources import KollaKubernetesResources


def usage():
    print("secret-generator.py requires one of these two commands: \
           create or delete")
    return

if len(sys.argv) != 2:
    usage()
    exit(1)

command = sys.argv[1].lower().strip()

if (command != 'create' and command != 'delete'):
    usage()
    exit(2)

password_file = "/etc/kolla/passwords.yml"

if not os.path.exists(password_file):
    print("You need to generate password file before using this script")
    exit(3)

with open(password_file, 'r') as stream:
    try:
        passwords = yaml.safe_load(stream)
    except yaml.YAMLError as exc:
        print(exc)
for element in passwords:
    if isinstance(passwords[element], six.string_types):
        service_name = element.replace('_', '-')
        password_value = passwords[element]
        nsname = 'kolla_kubernetes_namespace'
        nsname = KollaKubernetesResources.GetJinjaDict()[nsname]
        if command == "create":
            command_line = 'kubectl create secret generic {} {}{} {}{}'.format(
                           service_name,
                           " --from-literal=password=",
                           password_value,
                           "--namespace=",
                           nsname)
        else:
            command_line = "kubectl delete secret {} --namespace={}".format(
                           service_name, nsname)
        try:
            res = subprocess.check_output(
                command_line, shell=True,
                executable='/bin/bash')
            res = res.strip()  # strip whitespace
            print(res)
        except subprocess.CalledProcessError as e:
            print(e)
exit(0)



#!/usr/bin/env python
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
from selenium.common.exceptions import TimeoutException
from selenium.common.exceptions import WebDriverException
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait
import time


class Instances(object):

    def getDriver(self):
        count = 0
        while(True):
            try:
                ce = "http://%s:4444/wd/hub" % os.environ["HUB"]
                driver = webdriver.Remote(
                    command_executor=ce,
                    desired_capabilities={
                        "browserName": os.environ.get("browser", "firefox"),
                        "platform": "Linux"
                    }
                )
                return driver
            except WebDriverException as e:
                s = "%s" % e
                print("Got exception %s" % s)
                print("%s" % dir(s))
                if "Empty pool of VM for setup Capabilities" not in s:
                    raise
                time.sleep(5)
            if count == 60:
                raise Exception("Time out trying to get a browser")
            count += 1

    def setUp(self):
        self.driver = self.getDriver()
        print("Got driver")
        self.driver.get(os.environ["OS_HORIZON"])
        try:
            delay = 5
            element_present = EC.presence_of_element_located((By.NAME,
                                                              'username'))
            WebDriverWait(self.driver, delay).until(element_present)
            self.driver.title.index("Login - OpenStack Dashboard")
            elem = self.driver.find_element_by_name("username")
            elem.send_keys(os.environ["OS_USERNAME"])
            elem = self.driver.find_element_by_name("password")
            elem.send_keys(os.environ["OS_PASSWORD"])
            elem.send_keys(Keys.RETURN)
        except TimeoutException:
            print("Loading took too much time!")
            raise
        time.sleep(2)

    def testInstances(self):
        driver = self.driver
        driver.get("%s/project/instances/" % os.environ["OS_HORIZON"])
        try:
            delay = 5
            s = 'Instances - OpenStack Dashboard'
            element_present = EC.title_contains(s)
            WebDriverWait(self.driver, delay).until(element_present)
            print("Ok")
        except TimeoutException:
            print("Loading took too much time!")
            raise

    def screenshot(self):
        if self.driver:
            screenshot = os.path.join(os.environ['WORKSPACE'],
                                      'logs', 'horizon.png')
            self.driver.get_screenshot_as_file(screenshot)

    def tearDown(self):
        if self.driver:
            self.driver.close()
            self.driver.quit()

if __name__ == "__main__":
    i = Instances()
    try:
        i.setUp()
        i.testInstances()
    except Exception as e:
        print("Tearing down due to error. %s" % e)
        try:
            i.screenshot()
            i.tearDown()
        except Exception as e:
            print("Got another exception. %s" % e)
            pass
        pass
    else:
        i.tearDown()



#!/usr/bin/env python
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import errno
import os
import sys
import yaml


def _isdir(path, entry):
    return os.path.isdir(os.path.join(path, entry))


def merge_dict(a, b):
    for key in b:
        if key not in a:
            a[key] = b[key]
        else:
            if isinstance(b[key], dict):
                merge_dict(a[key], b[key])
            else:
                a[key] = b[key]


def main():
    path = os.path.abspath(os.path.dirname(sys.argv[0]))

    exampledir = os.path.join(path, "..", "examples")
    try:
        os.makedirs(exampledir)
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise

    srcdir = os.path.join(path, "..", "helm")
    microdir = os.path.join(srcdir, "microservice")
    microservices = os.listdir(microdir)
    values = {}

    for package in [p for p in microservices if _isdir(microdir, p)]:
        values_file = os.path.join(microdir, package, "values.yaml")
        with open(values_file, "r") as f:
            package_values = yaml.safe_load(f)
            merge_dict(values, package_values)

    # Remove some package specific values:
    if 'type' in values:
        del values['type']
    if 'element_name' in values['global']:
        del values['global']['element_name']

    y = yaml.safe_dump(values, default_flow_style=False)
    with open(os.path.join(exampledir, "cloud.yaml"), "w") as f:
        f.write("\n".join(["#%s" % l for l in y.split("\n")]))


if __name__ == '__main__':
    sys.exit(main())



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from kolla_kubernetes.pathfinder import PathFinder
from kolla_kubernetes.tests import base


class FindBaseDirTest(base.BaseTestCase):

    def test_find_installed_root(self):
        d = PathFinder.find_installed_root()

        # check for non-null
        self.assertIsNotNone(d)

        # check that project_root is not empty
        self.assertGreater(len(d), 0)

    def test_find_development_root(self):
        d = PathFinder.find_development_root()

        # check for non-null
        self.assertIsNotNone(d)

        # check that project_root is not empty
        self.assertGreater(len(d), 0)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import copy
import json
import os
import subprocess
import sys
import tempfile
import yaml

from kolla_kubernetes.commands import base_command
from kolla_kubernetes import pathfinder
from kolla_kubernetes import service_resources
from kolla_kubernetes import utils

KKR = service_resources.KollaKubernetesResources.Get()


class ResourceBase(base_command.KollaKubernetesBaseCommand):
    """Create, delete, or query status for kolla-kubernetes resources"""

    def get_parser(self, prog_name, skip_action=False):
        parser = super(ResourceBase, self).get_parser(prog_name)
        if not skip_action:
            parser.add_argument(
                "action",
                metavar="<action>",
                help=("One of [%s]" % ("|".join(service_resources.Service.
                                                VALID_ACTIONS)))
            )
        parser.add_argument(
            "resource_type",
            metavar="<resource-type>",
            help=("One of [%s]" % ("|".join(service_resources.Service.
                                            VALID_RESOURCE_TYPES)))
        )
        return parser

    def validate_args(self, args, skip_action=False):
        if not skip_action and args.action not in service_resources.Service.\
                VALID_ACTIONS:
            msg = ("action [{}] not in valid actions [{}]".format(
                args.action,
                "|".join(service_resources.Service.VALID_ACTIONS)))
            raise Exception(msg)
        if args.resource_type not in service_resources.Service.\
                VALID_RESOURCE_TYPES:
            msg = ("resource_type [{}] not in valid resource_types [{}]"
                   .format(args.resource_type,
                           "|".join(service_resources.Service.
                                    VALID_RESOURCE_TYPES)))
            raise Exception(msg)


class ResourceTemplate(ResourceBase):
    """Jinja process kolla-kubernetes resource template files"""

    # This command adds the CLI params as part of the Jinja vars for processing
    # templates. This is needed because some of the templates will need to know
    # the arguments with which this CLI is called.  For example, some
    # resource-type "disk" templates may reference '{{
    # kolla_kubernetes.cli.args.action }}' to produce output such as "gcloud
    # disk create" or "gcloud disk delete" based on the CLI params.  Most
    # templates will not require this, but it is needed for some.

    def get_parser(self, prog_name, skip_action=False):
        parser = super(ResourceTemplate, self).get_parser(prog_name,
                                                          skip_action)
        parser.add_argument(
            "resource_name",
            metavar="<resource-name>",
            nargs='+',
            help=("The unique resource-name under service->resource_type")
        )
        parser.add_argument(
            '--print-jinja-vars',
            action='store_true',
            help=("If this boolean is set, the final jinja vars dict used as"
                  " input for template processing will be printed to stderr.  "
                  " The vars dict is created by merging configuration files "
                  " from several sources before applying the dict to itself.")
        ),
        parser.add_argument(
            "-o",
            "--output",
            metavar="output",
            default="yaml",
            help=("Format output into one of [%s]" % (
                "|".join(['yaml', 'json'])))
        ),
        parser.add_argument(
            "-d",
            "--debug-container",
            metavar="container",
            dest='debug_container',
            action='append',
            help=("Assist in the debugging of the specified container")
        ),
        parser.add_argument(
            '--print-jinja-keys-regex',
            metavar='<print-jinja-keys-regex>',
            type=str,
            default=None,
            help=("If this regex string is set, all matching keys encountered"
                  " during the creation of the jinja vars dict will be printed"
                  " to stderr at each stage of processing.  The vars dict is"
                  " created by merging configuration files from several"
                  " sources before applying the dict to itself.")
        )
        return parser

    def take_action(self, args, skip_and_return=False):
        # Validate input arguments
        self.validate_args(args)

        multi = len(args.resource_name) != 1
        multidoc = {
            'apiVersion': 'v1',
            'kind': 'List',
            'items': []
        }

        if args.resource_name[0] == 'all':
            services = KKR.getServices()
            for service in services.keys():
                service_object = services.get(service)
                service_object.do_apply(args.action, args.resource_type)
            return

        for resource_name in args.resource_name:
            service_name = KKR.getServiceNameByResourceTypeName(
                args.resource_type,
                resource_name)
            service = KKR.getServiceByName(service_name)
            rt = service.getResourceTemplateByTypeAndName(
                args.resource_type, resource_name)

            tmpargs = copy.deepcopy(vars(args))
            tmpargs['resource_name'] = resource_name
            variables = KKR.GetJinjaDict(service_name, tmpargs,
                                         args.print_jinja_keys_regex)

            # Merge the template vars with the jinja vars before processing
            variables['kolla_kubernetes'].update(
                {"template": {"vars": rt.getVars()}})

            # handle the debug option --print-jinja-vars
            if args.print_jinja_vars is True:
                print(utils.YamlUtils.yaml_dict_to_string(variables),
                      file=sys.stderr)

            if args.resource_type == 'configmap' and \
               rt.getTemplate() == 'auto':
                nsname = 'kolla_kubernetes_namespace'
                cmd = "kubectl create configmap {} -o yaml --dry-run"
                cmd = cmd.format(resource_name)

                for f in pathfinder.PathFinder.find_config_files(
                        resource_name):
                    cmd += ' --from-file={}={}'.format(
                        os.path.basename(f).replace("_", "-"), f)

                # Execute the command
                out, err = utils.ExecUtils.exec_command(cmd)
                y = yaml.safe_load(out)
                y['metadata']['namespace'] = variables[nsname]

                res = y
            else:
                # process the template
                raw_doc = utils.JinjaUtils.render_jinja(
                    variables,
                    utils.FileUtils.read_string_from_file(
                        rt.getTemplatePath()))
                res = yaml.safe_load(raw_doc)

            if args.debug_container is not None:
                y = res
                kind = y['kind']
                if kind not in ('PetSet', 'Deployment', 'Job', 'DaemonSet',
                                'ReplicationController', 'Pod'):
                    raise Exception("Template doesn't have containers.")
                pod = y
                if kind != 'Pod':
                    pod = y['spec']['template']
                alpha_init_containers = None
                annotation = 'pod.alpha.kubernetes.io/init-containers'
                if 'metadata' in pod and 'annotations' in pod['metadata'] and \
                   annotation in pod['metadata']['annotations']:
                    j = json.loads(pod['metadata']['annotations'][annotation])
                    alpha_init_containers = {}
                    for c in j:
                        alpha_init_containers[c['name']] = c
                containers = {}
                for c in pod['spec']['containers']:
                    containers[c['name']] = c
                for c in args.debug_container:
                    found = False
                    warn_msg = "WARNING: container [{}] already has a" + \
                               " command override."
                    warn_msg = warn_msg.format(c)
                    if alpha_init_containers and c in alpha_init_containers:
                        if 'command' in alpha_init_containers[c]:
                            print(warn_msg, file=sys.stderr)
                        if 'args' in alpha_init_containers[c]:
                            del alpha_init_containers[c]['args']
                        alpha_init_containers[c]['command'] = \
                            ['/bin/bash', '-c',
                             'while true; do sleep 1000; done']
                        found = True
                    if c in containers:
                        if 'command' in containers[c]:
                            print(warn_msg, file=sys.stderr)
                        if 'args' in containers[c]:
                            del containers[c]['args']
                        containers[c]['command'] = \
                            ['/bin/bash', '-c',
                             'while true; do sleep 1000; done']
                        found = True

                    if not found:
                        raise Exception("Failed to find container: %s" % c)

                if alpha_init_containers:
                    annotation = 'pod.alpha.kubernetes.io/init-containers'
                    v = alpha_init_containers.values()
                    pod['metadata']['annotations'][annotation] = json.dumps(v)
            multidoc['items'].append(res)

        if skip_and_return:
            if multi:
                return yaml.safe_dump(multidoc)
            else:
                return yaml.safe_dump(res)

        if args.output == 'json':
            if multi:
                print(json.dumps(multidoc, indent=4), end="")
            else:
                print(json.dumps(res, indent=4), end="")
        elif multi:
            print(yaml.safe_dump(multidoc))
        else:
            if args.debug_container is not None:
                print(yaml.safe_dump(res), end="")
            else:
                print(raw_doc, end="")


class Template(ResourceTemplate):
    """Jinja process kolla-kubernetes resource template files"""

    def get_parser(self, prog_name):
        parser = super(Template, self).get_parser(prog_name,
                                                  skip_action=True)
        return parser

    def validate_args(self, args):
        super(Template, self).validate_args(args, skip_action=True)


class Resource(ResourceTemplate):
    """Create kolla-kubernetes resources"""

    def validate_args(self, args):
        super(Resource, self).validate_args(args)
        if args.action not in ['create', 'delete', 'status']:
            msg = ("action [{}] currently not supported".format(
                args.action))
            raise Exception(msg)

    def _kind_to_cli(self, kind):
        kind_map = {
            'PetSet': 'petset',
            'StatefulSet': 'statefulset',
            'Pod': 'pod',
            'ReplicationController': 'rc',
            'DaemonSet': 'daemonset',
            'Job': 'job',
            'Deployment': 'deployment',
            'ConfigMap': 'configmap',
            'Secret': 'secret',
            'Service': 'svc',
            'PersistentVolume': 'pv',
            'PersistentVolumeClaim': 'pvc',
        }
        if kind not in kind_map:
            msg = ("unknown template kind [{}].".format(kind))
            raise Exception(msg)
        return kind_map[kind]

    def _process_template(self, kind, namespace, template, names, action):
        nsflag = ""
        if kind != 'pv':
            nsflag = " --namespace={}".format(namespace)
        if action == 'create':
            with tempfile.NamedTemporaryFile() as tf:
                tf.write(template)
                tf.flush()
                s = "kubectl {} -f {}{}".format(
                    action, tf.name, nsflag)
                subprocess.call(s, shell=True)
        elif action == "delete":
            s = "kubectl delete {} {}{}".format(
                kind, names, nsflag)
            subprocess.call(s, shell=True)
        elif action == 'status':
            s = "kubectl get {} {}{}".format(
                kind, names, nsflag)
            subprocess.call(s, shell=True)

    def _get_ns(self, y):
        template_ns = ''
        try:
            template_ns = y['metadata']['namespace']
        except Exception:
            pass
        return template_ns

    def take_action(self, args):
        tmpl = super(Resource, self).take_action(args, skip_and_return=True)
        y = yaml.safe_load(tmpl)
        kind = y['kind']
        if kind == 'List':
            first_item = y['items'][0]
            ns = self._get_ns(first_item)
            type_map = {}
            for item in y['items']:
                if self._get_ns(item) != ns:
                    msg = "Bad template in list. Different namespaces."
                    raise Exception(msg)
                kind_cli = self._kind_to_cli(item['kind'])
                t = type_map.get(kind_cli)
                if t is None:
                    type_map[kind_cli] = t = []
                t.append(item['metadata']['name'])
            if args.action in ('status', 'delete'):
                for (kind_cli, names) in type_map.items():
                    self._process_template(kind_cli, ns, tmpl, ' '.join(names),
                                           args.action)
            else:
                self._process_template(self._kind_to_cli(first_item['kind']),
                                       ns, tmpl, '', args.action)

        else:
            ns = self._get_ns(y)
            self._process_template(self._kind_to_cli(kind),
                                   ns,
                                   tmpl, y['metadata']['name'],
                                   args.action)


class ResourceMap(base_command.KollaKubernetesBaseCommand):
    """List available kolla-kubernetes resources to be created or deleted"""

    # If the operator has any question on what Services have what resources,
    # and what resources reference which resourcefiles (on disk), then this
    # command is helpful.  This command prints the available resources in a
    # tree of Service->ResourceType->ResourceFiles.

    def get_parser(self, prog_name):
        parser = super(ResourceMap, self).get_parser(prog_name)
        parser.add_argument(
            "--resource-type",
            metavar="<resource-type>",
            help=("Filter by one of [%s]" % (
                "|".join(service_resources.Service.VALID_RESOURCE_TYPES)))
        )
        parser.add_argument(
            "--service-name",
            metavar="<service-name>",
            help=("Filter by one of [%s]" % (
                "|".join(KKR.getServices().keys())))
        )
        parser.add_argument(
            "-o",
            "--output",
            metavar="output",
            default="text",
            help=("Format output into one of [%s]" % (
                "|".join(['txt', 'json', 'yaml'])))
        )
        return parser

    def take_action(self, args):
        resources = []
        for service_name, s in KKR.getServices().items():

            # Skip specific services if the user has defined a filter
            if (args.service_name is not None) and (
                    args.service_name != service_name):
                continue

            if args.output == 'text':
                print('service[{}]'.format(s.getName()))

            for t in service_resources.Service.VALID_RESOURCE_TYPES:
                # Skip specific resource_types if the user has defined a filter
                if args.resource_type is not None and args.resource_type != t:
                    continue

                resourceTemplates = s.getResourceTemplatesByType(t)

                if args.output == 'text':
                    print('  resource_type[{}] num_items[{}]'.format(
                        t, len(resourceTemplates)))

                # Print the resource files
                for rt in s.getResourceTemplatesByType(t):
                    if args.output == 'text':
                        print('    ' + str(rt))
                    resources.append({
                        'resource_type': t,
                        'service_name': service_name,
                        'resource_name': rt.getName(),
                        'template': rt.getTemplate(),
                        'vars': rt.getVars(),
                    })

        if args.output == 'json':
            print(json.dumps(resources))
        if args.output == 'yaml':
            print(yaml.safe_dump(resources))



#!/usr/bin/env python

# Copyright 2015 Sam Yaple
# Copyright 2017 99Cloud Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import collections
import inspect
import os

from ansible.plugins import action
from six import StringIO

from oslo_config import iniparser


class OverrideConfigParser(iniparser.BaseParser):

    def __init__(self):
        self._cur_sections = collections.OrderedDict()
        self._sections = collections.OrderedDict()
        self._cur_section = None

    def assignment(self, key, value):
        cur_value = self._cur_section.get(key)
        if len(value) == 1 and value[0] == '':
            value = []
        if not cur_value:
            self._cur_section[key] = [value]
        else:
            self._cur_section[key].append(value)

    def parse(self, lineiter):
        self._cur_sections = collections.OrderedDict()
        super(OverrideConfigParser, self).parse(lineiter)

        # merge _cur_sections into _sections
        for section, values in self._cur_sections.items():
            if section not in self._sections:
                self._sections[section] = collections.OrderedDict()
            for key, value in values.items():
                self._sections[section][key] = value

    def new_section(self, section):
        cur_section = self._cur_sections.get(section)
        if not cur_section:
            cur_section = collections.OrderedDict()
            self._cur_sections[section] = cur_section
        self._cur_section = cur_section
        return cur_section

    def write(self, fp):
        def write_key_value(key, values):
            for v in values:
                if not v:
                    fp.write('{} =\n'.format(key))
                for index, value in enumerate(v):
                    if index == 0:
                        fp.write('{} = {}\n'.format(key, value))
                    else:
                        fp.write('{}   {}\n'.format(len(key) * ' ', value))

        def write_section(section):
            for key, values in section.items():
                write_key_value(key, values)

        for section in self._sections:
            fp.write('[{}]\n'.format(section))
            write_section(self._sections[section])
            fp.write('\n')


class ActionModule(action.ActionBase):

    TRANSFERS_FILES = True

    def read_config(self, source, config):
        # Only use config if present
        if os.access(source, os.R_OK):
            with open(source, 'r') as f:
                template_data = f.read()
            result = self._templar.template(template_data)
            fakefile = StringIO(result)
            config.parse(fakefile)
            fakefile.close()

    def run(self, tmp=None, task_vars=None):

        if task_vars is None:
            task_vars = dict()
        result = super(ActionModule, self).run(tmp, task_vars)

        # NOTE(jeffrey4l): Ansible 2.1 add a remote_user param to the
        # _make_tmp_path function.  inspect the number of the args here. In
        # this way, ansible 2.0 and ansible 2.1 are both supported
        make_tmp_path_args = inspect.getargspec(self._make_tmp_path)[0]
        if not tmp and len(make_tmp_path_args) == 1:
            tmp = self._make_tmp_path()
        if not tmp and len(make_tmp_path_args) == 2:
            remote_user = (task_vars.get('ansible_user')
                           or self._play_context.remote_user)
            tmp = self._make_tmp_path(remote_user)

        sources = self._task.args.get('sources', None)
        extra_vars = self._task.args.get('vars', list())

        if not isinstance(sources, list):
            sources = [sources]

        temp_vars = task_vars.copy()
        temp_vars.update(extra_vars)

        config = OverrideConfigParser()
        old_vars = self._templar._available_variables
        self._templar.set_available_variables(temp_vars)

        for source in sources:
            self.read_config(source, config)

        self._templar.set_available_variables(old_vars)
        # Dump configparser to string via an emulated file

        fakefile = StringIO()
        config.write(fakefile)

        remote_path = self._connection._shell.join_path(tmp, 'src')
        xfered = self._transfer_data(remote_path, fakefile.getvalue())
        fakefile.close()

        new_module_args = self._task.args.copy()
        new_module_args.pop('vars', None)
        new_module_args.pop('sources', None)

        new_module_args.update(
            dict(
                src=xfered
            )
        )

        result.update(self._execute_module(module_name='copy',
                                           module_args=new_module_args,
                                           task_vars=task_vars,
                                           tmp=tmp))
        return result



#!/usr/bin/env python
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from kubernetes import client
from kubernetes import config
import sys
import time


def usage():
    print("wait_for_pods.py requires three arguments.  a namespace name \
           a list of comma separated pod prefixes to monitor and a \
           comma separated list of valid end states to wait for \
           (such as completed and running)")
    return

if len(sys.argv) != 4:
    usage()
    exit(1)

namespace = sys.argv[1]
prefix_list = sys.argv[2].lower().split(',')
end_status_list = sys.argv[3].lower().split(',')

config.load_kube_config()
v1 = client.CoreV1Api()

done = False
while not done:
    matches = 0
    finished = 0
    # sleep at the start to give pods time to exist before polling
    time.sleep(5)
    kolla_pods = v1.list_namespaced_pod(namespace)
    for pod in kolla_pods.items:
        pod_name = pod.metadata.name.lower()
        pod_status = pod.status.phase.lower()
        for prefix in prefix_list:
            if pod_name.startswith(prefix):
                matches += 1
                if pod_status in end_status_list:
                    finished += 1

    if matches == finished:
        done = True
    else:
        print('Waiting for pods to be ready. Total: ' + str(matches) +
              ' Ready:' + str(finished))
